from __future__ import annotations

import logging
import time

import requests

logger = logging.getLogger(__name__)


class Sgis:
    """통계지리정보서비스 SGIS"""

    def __init__(
        self,
        api_key: str,
        api_sec: str,
    ) -> None:
        self.api_key: str = api_key
        self.api_sec: str = api_sec

    @staticmethod
    def raise_for_err_cd(parsed: dict) -> None:
        err_cd = parsed.get("errCd", 0)
        if err_cd:
            raise ValueError(f"[{err_cd}] {parsed.get('errMsg', 0)}")

    @property
    def access_token(self) -> str:
        if not hasattr(self, "_token") or int(self._timeout) / 1000 - 10 < time.time():
            self.auth()
        return self._token

    def auth(self) -> dict:
        # https://sgis.kostat.go.kr/developer/html/newOpenApi/api/dataApi/basics.html#auth
        url = "https://sgisapi.kostat.go.kr/OpenAPI3/auth/authentication.json"
        params = dict(consumer_key=self.api_key, consumer_secret=self.api_sec)
        resp = requests.get(url, params=params)
        parsed = resp.json()
        self.raise_for_err_cd(parsed)

        result = parsed.get("result", {})
        self._timeout = result.get("accessTimeout", 0)
        self._token = result.get("accessToken", "")
        return result

    def hadm_area(
        self,
        adm_cd: str = None,
        low_search: str = "1",
        year: str = "2023",
        session: requests.Session = None,
    ) -> dict:
        # https://sgis.kostat.go.kr/developer/html/newOpenApi/api/dataApi/addressBoundary.html#hadmarea
        # UTM-K (EPSG 5179)
        url = "https://sgisapi.kostat.go.kr/OpenAPI3/boundary/hadmarea.geojson"
        params = dict(
            accessToken=self.access_token,
            adm_cd=adm_cd,
            low_search=low_search,
            year=year,
        )
        resp = (
            session.get(url, params=params)
            if session
            else requests.get(url, params=params)
        )
        parsed = resp.json()
        self.raise_for_err_cd(parsed)
        return parsed

    def geocode_wgs84(
        self,
        address: str,
        page: int = 0,
        limit: int = 5,
        session: requests.Session = None,
    ) -> list[dict]:
        """입력된 주소 위치 제공 API(좌표계:WGS84, EPSG:4326)

        Args:
            address (str): 검색주소
            page (int, optional): 페이지. Defaults to 0.
            limit (int, optional): 결과 수. Defaults to 5.
            session (requests.Session, optional): 세션. Defaults to None.

        Returns:
            list[dict]: 검색결과
        """
        # https://sgis.kostat.go.kr/developer/html/newOpenApi/api/dataApi/addressBoundary.html#geocodewgs84
        url = "https://sgisapi.kostat.go.kr/OpenAPI3/addr/geocodewgs84.json"
        params = dict(
            accessToken=self.access_token,
            address=f"{address}",
            pagenum=f"{page}",
            resultcount=f"{limit}",
        )
        resp = (
            session.get(url, params=params)
            if session
            else requests.get(url, params=params)
        )
        parsed: dict = resp.json()
        self.raise_for_err_cd(parsed)

        result: dict = parsed.get("result", {})
        return result.get("resultdata", [])

    def geocode_utmk(
        self,
        address: str,
        page: int = 0,
        limit: int = 5,
        session: requests.Session = None,
    ) -> list[dict]:
        """입력된 주소 위치 제공 API(좌표계:UTM-K, EPSG:5179)

        Args:
            address (str): 검색주소
            page (int, optional): 페이지. Defaults to 0.
            limit (int, optional): 결과 수. Defaults to 5.
            session (requests.Session, optional): 세션. Defaults to None.

        Returns:
            list[dict]: 검색결과
        """
        # https://sgis.kostat.go.kr/developer/html/newOpenApi/api/dataApi/addressBoundary.html#geocode
        url = "https://sgisapi.kostat.go.kr/OpenAPI3/addr/geocode.json"
        params = dict(
            accessToken=self.access_token,
            address=f"{address}",
            pagenum=f"{page}",
            resultcount=f"{limit}",
        )
        resp = (
            session.get(url, params=params)
            if session
            else requests.get(url, params=params)
        )
        parsed: dict = resp.json()
        self.raise_for_err_cd(parsed)

        result: dict = parsed.get("result", {})
        return result.get("resultdata", [])



from __future__ import annotations

import logging

import requests

logger = logging.getLogger(__name__)


class Jusogokr:
    """도로명 주소 API"""

    def __init__(
        self,
        api_key: str,
    ) -> None:
        self.api_key: str = api_key

    @staticmethod
    def raise_for_common(common: dict):
        code = common.get("errorCode", "0")
        if code != "0":
            raise ValueError(f'[{code}] {common.get("errorMessage", "")}')

    def addr(
        self,
        keyword: str,
        page: int = 1,
        limit: int = 10,
        history: str = "N",
        sort: str = "none",
        detail: str = "N",
    ) -> list[dict]:
        # https://business.juso.go.kr/addrlink/openApi/searchApi.do
        url = "https://business.juso.go.kr/addrlink/addrLinkApi.do"
        params = dict(
            confmKey=self.api_key,
            keyword=f"{keyword}",
            currentPage=f"{page}",
            countPerPage=f"{limit}",
            resultType="json",
            hstryYn=history,
            firstSort=sort,
            addInfoYn=detail,
        )
        resp = requests.get(url, params=params)
        parsed = resp.json()
        results = parsed.get("results", {})
        self.raise_for_common(results.get("common", {}))
        juso = results.get("juso", {})
        return juso



from __future__ import annotations

import base64
import hashlib
import hmac
import time
from typing import Dict, List

import requests


class NaverAd:
    """NAVER Search Ad API"""

    def __init__(self, api_key: str, api_sec: str, cust_id: str) -> None:
        self.api_key: str = api_key
        self.api_sec: str = api_sec
        self.cust_id: str = cust_id
        self.host_url = "https://api.searchad.naver.com"

    def signature(self, timestamp: int, method: str, url: str) -> bytes:
        message = f"{timestamp}.{method}.{url}"
        hash = hmac.new(self.api_sec.encode(), message.encode(), hashlib.sha256)
        return base64.b64encode(hash.digest())

    def get_header(self, method, url) -> Dict:
        timestamp = int(round(time.time() * 1000))
        return {
            "X-Timestamp": f"{timestamp}",
            "X-API-KEY": self.api_key,
            "X-Customer": self.cust_id,
            "X-Signature": self.signature(timestamp, method, url),
        }

    def keywords_tool(
        self,
        *,
        site_id: str = None,
        biztp_id: int = None,
        keywords: str = None,
        event: int = None,
        month: int = None,
        show_detail: bool = False,
    ) -> Dict[str, List]:
        # https://naver.github.io/searchad-apidoc/#/tags/RelKwdStat
        method = "GET"
        url = "/keywordstool"
        headers = self.get_header(method, url)
        params = {
            "showDetail": 1 if show_detail else 0,
            **({"siteId": site_id} if site_id else {}),
            **({"biztpId": biztp_id} if biztp_id else {}),
            **({"hintKeywords": keywords} if keywords else {}),
            **({"event": event} if event else {}),
            **({"month": month} if month else {}),
        }
        resp = requests.get(self.host_url + url, headers=headers, params=params)
        resp.raise_for_status()
        return resp.json()

    @staticmethod
    def biztp_cd() -> List[Dict]:
        import json
        import pathlib

        with open(pathlib.Path(__file__).parent / "assets" / "na_biztp.json") as fp:
            return json.load(fp)

    @staticmethod
    def event_cd() -> List[Dict]:
        # https://gist.github.com/naver-searchad/235202ffb08f9433b6f7cb10e45875f7#file-seasonal_event_code-md
        import json
        import pathlib

        with open(pathlib.Path(__file__).parent / "assets" / "na_event.json") as fp:
            return json.load(fp)
